package ihm;

import javax.swing.*;
import java.awt.*;
import java.util.Vector;
import java.awt.geom.*;

public class Network {
   private static int diameter = 48;
   
   private String    addr;
   private Vector    symbs;
   
   public Network (String address) {
      addr = address;
      symbs = new Vector();
   }

   private Point2D center () {
      if (symbs.size() > 0) {
         int x=0;
         int y=0;
         
         for (int i=0; i<symbs.size(); i++) {
            Point2D p = ((Symbole)symbs.elementAt(i)).getCenter();
            x += p.getX();
            y += p.getY();
         }
         
         return new Point2D.Double (x / symbs.size(), y / symbs.size());
      } else {
         return new Point2D.Double (0, 0);
      }
   }

   public void addSymb (Symbole s) {
      symbs.addElement (s);
   }
   
   public void removeSymb (Symbole s) {
      while (symbs.remove (s));
   }
   
   public String getAddr() {
      return addr;
   }
   
   public String toString () {
      return addr;
   }
   
   public void drawIt (Graphics2D g) {
      if (symbs.size() <= 0) {
         return;
      }
      Rectangle2D addrBounds = g.getFont().getStringBounds (addr, g.getFontRenderContext());
      Point2D c;
      if (symbs.size() == 1) {
         Rectangle r = ((Symbole)symbs.elementAt(0)).getBounds();
         c = new Point2D.Double (r.getX()+r.getWidth()/2, r.getY()+r.getHeight() + diameter/2 + 20);
      } else {
         c = center();
      }
      Ellipse2D e = new Ellipse2D.Double(c.getX()-diameter/2, c.getY()-diameter/2, diameter, diameter);
      g.setPaint (new Color (100, 100, 100));
      for (int i=0; i<symbs.size(); i++) {
         Point2D p = ((Symbole)symbs.elementAt(i)).getCenter();
         g.draw (new Line2D.Double (c.getX(), c.getY(), p.getX(), p.getY()));
      }
      g.setPaint (new Color (210, 190, 190));
      g.fill (e);
      g.setPaint (new Color (100, 100, 100));
      g.draw (e);
      g.setColor (new Color (200, 25, 25));         
      g.drawString (addr, (int)(c.getX()-diameter/2-(addrBounds.getWidth()-diameter)/2), (int)(c.getY()-diameter/2+diameter+addrBounds.getHeight()+3));      
   }  
}