package moteur;

import java.io.*;
import java.util.Date;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector; 

import ihm.IHMMoteur;
import ihm.SymbBounds;

import reseau.ReseauMoteur;
import reseau.MibNode;
import reseau.Mib;
import reseau.MibNodeNotFoundException;
import reseau.SnmpErrorException;

public class Moteur implements MoteurIHM, MoteurReseau {

    private String dtdreseau = "reseau.dtd";
    private String securite = "SNMPapp1.0";
    private Reseau reseau;
    private ElementReseauConfiguration [] tabERC = new ElementReseauConfiguration[ET.getTypesCount()];
    private boolean log;
    private String logFile = "app.log";

    private ReseauMoteur reseauMoteur;
    private IHMMoteur ihmMoteur;


    //-----
    // constructeur
    //-----
    public Moteur() {
        reseau = new Reseau();
        log = true;
        for(int i = 0; i < ET.getTypesCount(); i++)
        {
            tabERC [i] = new ElementReseauConfiguration();
            ElementReseau temp = new ElementReseau();
            temp.setType(i);
            tabERC[i].setElementReseau(temp);
        }
    }

    public void init(IHMMoteur ihmMoteur, ReseauMoteur reseauMoteur)
    {
        setIHMMoteur(ihmMoteur);
        setReseauMoteur(reseauMoteur);
    }

    //-----
    // setteurs
    //-----

    public void setIHMMoteur(IHMMoteur ihmMoteur)
    {
        this.ihmMoteur = ihmMoteur;
        ihmMoteur.init(this);
    }


    public void setReseauMoteur(ReseauMoteur reseauMoteur)
    {
        this.reseauMoteur = reseauMoteur;
        reseauMoteur.init(this);
    }

    public void setReseau(Reseau reseau)
    {
        this.reseau = reseau.clone();
    }

    public void setTabERC(ElementReseauConfiguration [] tabERC)
    {
        this.tabERC = tabERC;
    }

    public void setLog(boolean log)
    {
        this.log = log;
    }
    //-----
    // getteurs
    //-----

    public Reseau getReseau()
    {
        return reseau;
    }

    public ElementReseauConfiguration [] getTabERC()
    {
        return tabERC;
    }

    public boolean getLog()
    {
        return log;
    }
    
    
    public int getType(int id)
    {
         ElementReseau temp = reseau.elementWithId(id);
         if(temp != null)
         {
            return temp.getType();
         }
         return -1;
    }

    public String getName(int id)
    {
         ElementReseau temp = reseau.elementWithId(id);
         if(temp != null)
         {
            return temp.getNom();
         }
         return null; 
    }

    public String getIpContact(int id)
    {
         ElementReseau temp = reseau.elementWithId(id);
         if(temp != null)
         {
            return temp.getIpContact();
         }
         return null;
    }    
    
    public boolean getOnline(int id)
    {
         ElementReseau temp = reseau.elementWithId(id);
         if(temp != null)
         {
            return temp.getEtat();
         }
         return false;
    }        
    
	public boolean isOnline(int id)
	{
         ElementReseau temp = reseau.elementWithId(id);
         if(temp != null)
         {
            if(temp.getIpContact().equals(""))
               return false;
            else
               return reseauMoteur.isOnline(temp.getIpContact(), temp.getCommunity());
         }
         return false;	
	}
	
    //-----
    // methodes publics
    //-----

    //ouvre le fichier en parametre
    //renvoie false si le type de fichier n'est pas un type support par l'application
    public SymbBounds[] open(File fichier)
    {
        if((fichier.getName()).endsWith(new String(".xml")))
        {
            
            /*
             * traitement de l'ouverture d'un xml:
             *     on initialise le reseau avec les donne du xml
             *     on creer le vecteur d'ERC avec les proprietes par defaut
             *
             *     erreur possible (retourne false):
             *          le fichier ne respecte pas la dtd
             */
            
            reseau = XmlOutils.XmlFile2Reseau(fichier);
			
            if(reseau == null)
               return null;
            else
               return getAllElement();
        }
        else if((fichier.getName()).endsWith(new String(".sav")))
        {
           
            /*
             * traitement de l'ouverture d'un sav:
             *     on initialise le reseau avec les donne du xml definit dans le sav
             *     on creer le vecteur d'ERC avec les proprietes definit dans le sav
             *
             *     erreur possible (retourne false):
             *          le fichier ne contient pas xml associ
             *          le xml associ n'existe plus
             *          le fichier ne contient pas les bonnes donnes
             */
            try
            {
                String name = fichier.getAbsolutePath();
				ObjectInputStream ois = new ObjectInputStream(new
                        FileInputStream(fichier));
                String securite = (String)ois.readObject();
                if(securite.equals(this.securite))
                {
                    String ext = ".xml";
					String xmlAssoc = (name.substring(0,name.length()-ext.length())) + ext;
					System.out.println("ouverture de "+xmlAssoc);
               reseau = XmlOutils.XmlFile2Reseau(new File(xmlAssoc));
					Integer asPos = (Integer)ois.readObject();
					SymbBounds[] tab = null;
					if(asPos == 1)
						 tab = (SymbBounds[])ois.readObject(); 
               if(reseau == null)
                    {
                        reseau = new Reseau();
                        return null;
                    }
                    Integer nbERC = (Integer)ois.readObject();
                    tabERC = new ElementReseauConfiguration[nbERC];
                    for(int i = 0; i< nbERC; i++)
                        tabERC[i] = (ElementReseauConfiguration)ois.readObject();
                    return tab;
                }
                else
                {
                    
                    return null;
                }
            }
            catch(IOException ioe)
            {
                
                return null;
            }
            catch(ClassNotFoundException cnfe)
            {
                
                return null;
            }
        }
        else
        {
            
            return null;
        }
    }


    // sauvegarde dans un .sav et dans le .xml associ
    public String save(File fichier, SymbBounds[] positions)
    {
        
        /*
         * traitement de la sauvegarde:
         *     on sauvegarde les ERC et autres propriete graphique du reseau dans
         * le .sav, ainsi que l'adresse du xml associ
         *     on sauvegarde le reseau dans le xml du meme nom.
         *
         *
         *     erreur possible (retourne false):
         *          pas de reseau creer
         *          le fichier en parametre n'est pas un .sav
         *
         */
        if(fichier == null) return "fichier non renseign";
        String name = fichier.getAbsolutePath();
        if( name.endsWith(new String(".sav")) )
        {
            try
            {
                
				String ext = ".xml";
				String xmlAssoc = (name.substring(0,name.length()-ext.length())) + ext;
				System.out.println("sauvegarde de "+xmlAssoc);
                String mess = export2Xml(new File(xmlAssoc));
                if(mess.equals("true"))
                {
                  ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(fichier));
                  oos.writeObject(this.securite);
                  //oos.writeObject(xmlAssoc);
                  if(positions != null && positions.length>0)
                  {
                     oos.writeObject(new Integer(1));
                     oos.writeObject(positions);
                  }
                  else oos.writeObject(new Integer(-1));

                  Integer nbERC = new Integer(tabERC.length);
                  oos.writeObject(nbERC);
                  for (int i = 0; i < tabERC.length; i++)
                     oos.writeObject(tabERC[i]);
                  oos.flush();
                  return "true";
                }
                else return mess;
            }
            catch (IOException ioe)
            {
                
                return "erreur entre/sortie fichier";
            }
        }
        else
        {
            
            return "type de fichier non support =>  doit tre .sav";
        }
    }


    //export le reseau en xml
    public String export2Xml(File fichier)
    {
        
		
        if((fichier.getName()).endsWith(new String(".xml")))
        {
            System.out.println("export xml "+fichier.getAbsolutePath());
			if(XmlOutils.reseau2XmlFile(fichier, this.reseau, this.dtdreseau))
                return "true";
            else
                return "le fichier "+ ((String)fichier.getName()) + " n'a pas pu tre crer";
        }
        else
        {
            
            return "type de fichier non support =>  doit tre .xml";
        }
    }


    //ajoute un element au reseau
    /*
     * renvoie [ cle, id ] si ok
     * renvoie [ cle, -1 ] si pas ok
     *
     */
    public int addElementReseau(int type)
    {
        if(ET.getTypeName(type).equals(""))
        {
            return -1;
        }

        ElementReseau e = new ElementReseau();
        e.setType(type);
        int i = 0;
        while( reseau.containsElement(i) ) i ++;
        e.setId(i);
        reseau.addElement(e);

        return i;
    }


    //supprime un element du reseau
    /*
     * renvoie true si l'element est bien supprimer
     *         false si l'element n'existe pas
     *
     */
    public boolean removeElementReseau(int id)
    {
        if(reseau.containsElement(id))
        {
            reseau.removeElement(id);
            return true;
        }
        return false;
    }




    //retourne les propriet de base a afficher pour l'element
    public Hashtable<String, String[]> getBase(int id)
    {
        ElementReseau e = reseau.elementWithId(id);
        
        
        if (e != null)
        {
            ElementReseauConfiguration erc = tabERC[e.getType()];
			erc.setElementReseau(e);
            Hashtable<String, String[]> temp = erc.getBase();
            String ip = e.getIpContact();
            if (ip.equals(""))
            {
                return temp;
            }
            Enumeration enu  = temp.keys();
            while(enu.hasMoreElements())
            {
                String key = (String)enu.nextElement();
                String [] tab = (String [])temp.get(key);
                if( ! tab[2].equals(""))
                {

                    try
                    {
                        tab[1] = getValue(ip, e.getCommunity(), tab[2]);
                        tab[0] = ((Integer)getAccess(tab[2])).toString();
                        e.setEtat(true);
                    }
                    catch(IOException ioe)
                    {
                        tab[1] = "##ioerror##";
                        tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                        e.setEtat(false);
                    }
                    catch(SnmpErrorException see)
                    {
                        tab[1] = "##snmperror##";
                        tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                    }
                    catch(MibNodeNotFoundException mnfe)
                    {
                        tab[1] = "##mibnodenotfound##";
                        tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                    }
                }
            }
            erc.setElementReseau(e);
            return erc.getBase();
        }
        else return null;
    }

    //retourne les propriet personalis a afficher pour l'element
    public Hashtable<String, String[]> getPerso(int id)
    {
        ElementReseau e = reseau.elementWithId(id);
        if (e != null)
        {
            ElementReseauConfiguration erc = tabERC[e.getType()];
            String ip = e.getIpContact();

            erc.setElementReseau(e);
            Hashtable<String, String[]> temp = erc.getPerso();
            Enumeration enu = temp.keys();
            while (enu.hasMoreElements())
            {
                String key = (String) enu.nextElement();
                String tab[] = (String[]) temp.get(key);
                try {
                    if (!ip.equals("") && !tab[2].equals(""))
                    {
                        tab[1] = getValue(ip,e.getCommunity(), tab[2]);
                        tab[0] = ((Integer) getAccess(tab[2])).toString();
                        e.setEtat(true);
                    }
                    else
                    {
                        tab[1] = "##erreur adresse ou code Mib##";
                        tab[0] = "-1";
                        e.setEtat(false);
                    }
                } catch (IOException ioe) {
                    tab[1] = "##ioerror##";
                    tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                    e.setEtat(false);
                } catch (SnmpErrorException see) {
                    tab[1] = "##snmperror##";
                    tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                }
                catch(MibNodeNotFoundException mnfe)
                {
                    tab[1] = "##mibnodenotfound##";
                    tab[0] = ((Integer)Mib.ACCESS_RO).toString();
                }

            }
            return temp;
        }
        else return null;
    }

    //active les logs
    public void logOn()
    {
        this.log = true;
    }

    //desactive les logs
    public void logOff()
    {
        this.log = false;
    }


    //personalisation de l'affichage des proprietes
    public String addProprietePerso(int id, String key, String[] value)
    {
        if( value.length == 3)
        {
            if(reseau.containsElement(id))
            {
                ElementReseau temp = reseau.elementWithId(id);
                /*appel reseau*/
                tabERC[temp.getType()].addElementPerso(key,value);
                return "true";
            }
            else return "l'element "+ id +" n'existe pas";

        }
        else return "valeur invalide => doit tre un String [] de taille 3";
    }

    public String replaceProprietePerso(int id, String key, String [] old, String[] value)
    {
        if( value.length == 3)
        {
            if(reseau.containsElement(id))
            {
                ElementReseau temp = reseau.elementWithId(id);
                if(!value[2].equals(""))
                 {
                     String ip = temp.getIpContact();
                     if(ip == null)
                         return "pas d'interface";
                     try
                     {
                        setValue(ip,temp.getCommunity(),value[2],value[1]);
                        temp.setEtat(true);
                     }
                     catch (IOException ioe) { temp.setEtat(false); return "ioe";}
                     catch (SnmpErrorException see) {return "see"; }
                     catch(MibNodeNotFoundException mnfe){ return "mnfe"; }
                }
                tabERC[temp.getType()].setElementPerso(key,value);
                return "true";
            }
            else return "l'element "+ id +" n'existe pas";

        }
        else return "valeur invalide => doit tre un String [] de taille 3";
    }

    //personalisation de l'affichage des proprietes
    public String removeProprietePerso(int id, String key)
    {
        if(reseau.containsElement(id))
        {
            ElementReseau temp = reseau.elementWithId(id);
            tabERC[temp.getType()].removeElementPerso(key);
            return "true";
        }
        else return "l'element "+ id +" n'existe pas";
    }

    //personalisation de l'affichage des proprietes
    public String addProprieteBase(int id, String key, String[] value)
    {
        if( value.length == 3)
        {
            if(reseau.containsElement(id))
            {
                ElementReseau temp = reseau.elementWithId(id);
                /*appel reseau*/
                if(key.startsWith("element"))
                {
                    try
                    {
                        Integer i = new Integer(value[1]);
                        temp.addElement(i);
                    }
                    catch(NumberFormatException nfe){ return "erreur l'element n'est pas un entier"; }
                }
                if(key.equals("ipContact"))
                {
                  if(reseau.elementWithIp(value[1]) == null)
                     temp.setIpContact(value[1]);
                }
                if(key.equals("community"))
                {
					temp.setCommunity(value[1]);
                }
                tabERC[temp.getType()].addElementBase(key, value);
                return "true";

            }
            else return "l'element "+ id +" n'existe pas";

        }
        else return "valeur invalide => doit tre un String [] de taille 3";
    }

    public String replaceProprieteBase(int id, String key, String [] old, String[] value)
    {
        if( value.length == 3)
        {
            if(reseau.containsElement(id))
            {
                ElementReseau temp = reseau.elementWithId(id);
                if(key.startsWith("element"))
                {
                    try
                    {
                        Integer i = new Integer(value[1]);
                        Integer o = new Integer(old[1]);
                        int index = temp.indexOfElement(o);
                        if(index < 0)
                            temp.addElement(i);
                        else
                            temp.setElement(i,index);
                    }
                    catch(NumberFormatException nfe){ return "erreur l'element n'est pas un entier"; }
                }
                if(key.equals("ipContact"))
                {
                    if(reseau.elementWithIp(value[1]) == null)
                        temp.setIpContact(value[1]);

                }
                if(key.equals("type"))
                {
                    String s = value[1];
                    try
                    {
                        Integer i = new Integer(s);
                        if(i < 0 && i >= ET.getTypesCount()) return "type invalide";
                    }
                    catch(NumberFormatException nfe){ return "type invalide"; }
                    temp.setType(new Integer(s));
                }
                if(key.equals("nom"))
                {
                    String s = value[1];
                    temp.setNom(s);
                }
                if(key.equals("community"))
                {
                    String s = value[1];
                    temp.setCommunity(s);
                }
                if(key.equals("etat"))
                {
                    String s = value[1];
                    temp.setEtat(Boolean.parseBoolean(s));
                }

                if(!value[2].equals(""))
                {
                    String ip = temp.getIpContact();
                    if(ip.equals(""))
                        return "pas d'interface";
                    try
                    {
                        setValue(ip,temp.getCommunity(),value[2],value[1]);
                        temp.setEtat(true);
                    }
                    catch (IOException ioe) { temp.setEtat(false); return "ioe";}
                    catch (SnmpErrorException see) {return "see"; }
                    catch(MibNodeNotFoundException mnfe){ return "mnfe"; }
                }

                tabERC[temp.getType()].setElementBase(key, value);
                return "true";

            }
            else return "l'element "+ id +" n'existe pas";

        }
        else return "valeur invalide => doit tre un String [] de taille 3";
    }


    //personalisation de l'affichage des proprietes
    public String removeProprieteBase(int id, String key, String [] value)
    {
        if(reseau.containsElement(id))
        {
            ElementReseau temp = reseau.elementWithId(id);
            if (key.startsWith("element")) {
                try
                {
                    Integer i = new Integer(value[1]);
                    temp.removeElement(i);
                } catch (NumberFormatException nfe) {
                    return "erreur l'element n'est pas un entier";
                }
            }
            if (key.equals("ipContact")) {
               temp.setIpContact("");
            }
            if (key.equals("community")) {
				temp.setCommunity("");
            }
            if (key.equals("type")) {
                return "impossible de supprimer le type";
            }
            if (key.equals("nom")) {
                temp.setNom("");
            }
            if (key.equals("etat")) {
                return "impossible de supprimer l'etat";
            }


            tabERC[temp.getType()].removeElementBase(key);
            return "true";
        }
        else return "l'element "+ id +" n'existe pas";
    }


    public String updateBase(int id, Hashtable<String, String[]> old, Hashtable<String, String[]> value)
    {
        Enumeration en = old.keys();
        String result = "true";
        while(en.hasMoreElements())
        {
            String key = (String)en.nextElement();

            if(value.containsKey(key))
            {
                String temp1 = (String)((String[])value.get(key))[1];
                String temp2 = (String)((String[])old.get(key))[1];
                
                
                if(! temp1.equals( temp2 ) )
                {

                    result = replaceProprieteBase(id, key, old.get(key),
                                                  value.get(key));
                    if (!result.equals("true"))return result;
                }
            }
            else
            {
                result = removeProprieteBase(id,key, old.get(key));
                if (! result.equals("true")) return result;
            }
        }

        en = value.keys();
        while(en.hasMoreElements())
        {

            String key = (String)en.nextElement();
            result = addProprieteBase(id,key,value.get(key));
            if (! result.equals("true")) return result;
        }

        return result;
    }

    public String updatePerso(int id, Hashtable<String, String[]> old, Hashtable<String, String[]> value)
       {
           Enumeration en = old.keys();
           String result = "true";
           while(en.hasMoreElements())
           {
               String key = (String)en.nextElement();

               if(value.containsKey(key))
               {
                   if(! ((String[])value.get(key))[1].equals( (String)((String[])old.get(key))[1] ) )
                   {
                       result = replaceProprietePerso(id, key, null, value.get(key));
                       if (!result.equals("true"))return result;
                   }
               }
               else
               {
                   result = removeProprietePerso(id,key);
                   if (! result.equals("true")) return result;
               }
           }

           en = value.keys();
           while(en.hasMoreElements())
           {

               String key = (String)en.nextElement();
               result = addProprietePerso(id,key,value.get(key));
               if (! result.equals("true")) return result;
           }

           return result;
       }


    public boolean deleteReseau()
    {
        reseau = new Reseau();
        return true;
    }


    public boolean deleteTabERC()
    {
        for(int i = 0; i<tabERC.length ; i++)
            tabERC[i].setPerso(new Hashtable());
        return true;
    }

    public String delete()
    {
        if( ! deleteReseau()) return "impossible de supprimer le reseau";
        if( ! deleteTabERC()) return "impossible de supprimer les ERCs";
        return "true";
    }

	
	public String getNetwork (int id)
	{
		String result = null;
		ElementReseau temp = reseau.elementWithId(id);
		if(temp != null)
		{
			String ip = temp.getIpContact();
			if(!ip.equals(""))
			{
				String oid = ".1.3.6.1.2.1.4.20.1.3."+ip;
                try 
				{
					String masque = getValue(ip,temp.getCommunity(), oid);
					//String masque = "255.255.255.0";
					//System.out.println("ip: "+ip);
					//System.out.println("mask: "+masque);
					StringTokenizer stip = new StringTokenizer(ip,".");
					StringTokenizer stm = new StringTokenizer(masque,".");
					if(stip.countTokens() == stm.countTokens())
					{
						try
						{
							int [] reseau = new int [stip.countTokens()];
							int i = 0;
							while(stip.hasMoreTokens())
							{
								reseau[i] = Integer.parseInt(stip.nextToken()) & Integer.parseInt(stm.nextToken());
								i++;
							}
							result = "";
							for(int j=0; j<(reseau.length - 1); j++)
								result = result + reseau[j] + ".";
							result = result + reseau[(reseau.length -1)];
						}
						catch( NumberFormatException nfe){ return null;  }
					}
					temp.setEtat(true);
                } 
				catch (IOException ioe) { temp.setEtat(false); return null; }
                catch (SnmpErrorException see) { return null;  }
                catch(MibNodeNotFoundException mnfe) { return null;  }

			}
		}
	
		return result;
	}

   

   
    //appel reseau

    public void setValue(String ip, String community, String oid, String value)throws java.io.IOException, SnmpErrorException, MibNodeNotFoundException
    {
        reseauMoteur.setRequest(ip,community, oid, value);
    }

    public String getValue(String ip, String community, String oid) throws java.io.IOException, SnmpErrorException, MibNodeNotFoundException
    {
            return reseauMoteur.getRequest(ip,community, oid);
    }


     public MibNode getNode(String codeMib)
     {
         try
         {
             MibNode temp = reseauMoteur.getNode(codeMib);
             return temp;
         }
         catch(MibNodeNotFoundException mnfe) { return null; }

     }


    public int getAccess(String codeMib)
    {
        try
        {
            MibNode temp = reseauMoteur.getNode(codeMib);
            return temp.getAccess();
        }
        catch(MibNodeNotFoundException mnfe) { return -1; }

    }

    //gestion de l'arbre

    public MibNode getRoot()
    {
        return reseauMoteur.getRoot();
    }


    public void setAlert(String ip, String message, String oid, Date time, int err1, int err2)
    {
        String messLog = ip+"|"+message+"|"+oid+"|"+err1+"|"+err2;
		ElementReseau temp = reseau.elementWithIp(ip);
      if(temp != null)
      {
         ihmMoteur.notifyAlert(temp.getId(), messLog, time);
		 if(log)
            writeLog("["+time+"] "+messLog);
         
      } else {
         if(log)
            writeLog("["+time+"]"+messLog + "|Element inconnu, l'alerte n'a pas t transmise!");
      }
      
    }

	public void endApp ()
	{
		ihmMoteur.finalize();
      reseauMoteur.finalize();
      System.out.println("application ferme");
		System.exit(0);
	}

	public void writeLog(String log)
	{
	    PrintWriter fos = null;
		try
	    {
			fos = new PrintWriter( new FileWriter (logFile, true));
			fos.println(log);
	    }
	    catch (FileNotFoundException e)
	    {
			try
			{
				fos = new PrintWriter( new FileWriter (logFile));
				fos.println(log);
			}
			catch (IOException ioe){}
	    }
		catch (IOException ioe){}
	    finally
	    {

	        fos.close();

	    }
	}
	
   
	public SymbBounds [] getAllElement()
	{
		if(reseau == null || reseau.nbElements() <=0) 
         return null;
         
		SymbBounds [] tab = new SymbBounds[reseau.nbElements()];
      
		for(int i = 0; i < reseau.nbElements(); i++)
		{
			ElementReseau temp = reseau.getElement(i);
			if(temp != null)
			{
				tab[i] = new SymbBounds(temp.getId());
			}
		}
		return tab;	

	}
	

}
