package reseau;

import java.util.Iterator;
import java.util.Vector;

public class CustomType {
	public static final int	INTEGER		= 1;
	public static final int	OCTETSTRING	= 2;
	public static final int	STRING		= 3;
	public static final int	OBJECT		= 4;
	public static final int SEQUENCE	= 30;
	public static final int	IPADDRESS	= 40;
	public static final int	COUNTER		= 41;
	public static final int	GAUGE		= 42;
	public static final int	TIMETICKS	= 43;
	public static final int	OPAQUE		= 44;
	public static final int	COUNTER64	= 46;

	private String	name;
	private int		type;
	private int		len;
	
	class SubType {
		private int index;
		private String name;
		private String type;
		public SubType (int i, String n, String t) {
			index = i;
			name = n;
			type = t;
			//System.out.println("Creation du SubType " + n + " : " + t);
		}
		public int getIndex() { return index; }
		public String getName() { return name; }
		public String getType() { return type; }
	}
	
	private int	nbSubTypes;
	private Vector<SubType>	subTypes;
	
	public CustomType (String n, int t) {
		name = n;
		type = t;
		len = 0;
		subTypes = new Vector<SubType>();
		nbSubTypes = 0;
		//System.out.println("Creation d'un CustomType : " + n);
	}
	
	public CustomType (String n, int t, int l) {
		name = n;
		type = t;
		len = 0;
		subTypes = new Vector<SubType>();
		nbSubTypes = 0;
	}
	
	public String getName() {
		return name;
	}
	
	public int getType() {
		return type;
	}
	
	public void addSubType(String n, String t) {
		nbSubTypes++;
		subTypes.add(new SubType(nbSubTypes, n,t));
	}
	
	public SubType getSubType(String name) throws MibNodeNotFoundException {
		int		n;
		SubType	s;
		try {
			n = Integer.parseInt(name);
			if (n <=0 || n > nbSubTypes) throw new MibNodeNotFoundException();
			return subTypes.get(n-1);
		} catch (NumberFormatException e) {
			Iterator<SubType>	i = subTypes.iterator();
			while (i.hasNext()) {
				s = i.next();
				if (s.getName().compareTo(name)==0) return s;
			}
		}
//		return subTypes.get(index);
		throw new MibNodeNotFoundException();
	}
	
	public boolean isSequence() {
		if (type == SEQUENCE)
			return true;
		else
			return false;
	}
	
	public Type getUsableType(String value) {
		Type	ret = null;
		
		switch (type) {
		case INTEGER:
			ret = new TypeInteger(len);
			ret.fromString(value);
			break;
		case OCTETSTRING:
			ret = new TypeOctetString();
			ret.fromString(value);
			break;
		case STRING:
			ret = new TypeString(value);
			break;
		case OBJECT:
			ret = new TypeObject(value);
			break;
		case COUNTER:
			ret = new TypeInteger(Type.COUNTER, 0, 4);
			ret.fromString(value);
			break;
		case GAUGE:
			ret = new TypeInteger(Type.GAUGE, 0, 4);
			ret.fromString(value);
			break;
		case TIMETICKS:
			ret = new TypeInteger(Type.TIMETICKS, 0, 4);
			ret.fromString(value);
			break;
		case COUNTER64:
			ret = new TypeInteger(Type.COUNTER64, 0, 8);
			ret.fromString(value);
			break;
		case IPADDRESS:
			ret = new TypeIpAddress();
			ret.fromString(value);
			break;
		}
		
		return ret;
	}
}
