package reseau;

import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

public class MibNode {
	public final static int		NODE			= 1;
	public final static int		LEAF			= 2;
	public final static int		VIRTUAL_NODE	= 3;
	public final static int		TAB_NODE		= 4;

	private	MibNode				parent;
	private	Vector<MibNode>		childs;

	private int					index;
	private	String				name;
	
	protected int				access;
	protected int				status;

	protected String			description;
	protected int				nodeType;
	protected String			type;
	
	private MibModule			module;
	
	private class MibVirtualNode extends MibNode {
		public MibVirtualNode(MibNode parent, int i, String n) {
			super (parent, i, n, module);
			nodeType = VIRTUAL_NODE;
			access = parent.access;
			status = parent.status;
			description = parent.description;
			type = parent.type;
			module = parent.module;
		}
	}
	
	private class MibTabNode extends MibNode {
		public MibTabNode(MibNode parent, int i, String n, String t) {
			super (parent, i, n, module);
			nodeType = TAB_NODE;
			access = parent.access;
			status = parent.status;
			description = parent.description;
			type = t;
			module = parent.module;
		}
	}
	
	public MibNode (MibNode p, int i, String n, MibModule mod) {
		parent	= p;
		index	= i;
		name	= n;
		childs	= new Vector<MibNode>();
		
		access		= Mib.ACCESS_NA;
		status		= Mib.STATUS_CURRENT;
		description	= "";
		
		nodeType	= NODE;
		module		= mod;
	}
	
	public MibNode (MibNode p, int i, String n, MibModule mod, int a, int s, String d) {
		parent	= p;
		index	= i;
		name	= n;
		childs	= new Vector<MibNode>();
		
		access		= a;
		status		= s;
		description	= d;
		type		= "";
		
		nodeType	= NODE;
		module		= mod;
	}
	
	public MibNode (MibNode p, int i, String n, MibModule mod, int a, int s, String d, String t) {
		parent	= p;
		index	= i;
		name	= n;
		childs	= new Vector<MibNode>();
		
		access		= a;
		status		= s;
		description	= d;
		type		= t;
		nodeType	= LEAF;
		module		= mod;
	}
	/*
	public void setModule(MibModule m) {
		module = m;
	}*/
	
	public boolean isAccessible() {
		if (access == Mib.ACCESS_RO || access == Mib.ACCESS_RW)
			return true;
		else
			return false;
	}
	
	public boolean isRoot() {
		if (parent == null) return true;
		else return false;
	}
	
	public String getType() {
		return type;
	}
	
	public String getName() {
		return name;
	}
	
	public int getIndex() {
		return index;
	}
	
    public int getNodeType() {
        return nodeType;
    }

    public int getStatus() {
        return status;
    }

	public int getAccess() {
		return access;
	}
	
	public String getDescription() {
		return description;
	}
	
	public boolean isLeaf() {
		if (nodeType == NODE)
			return false;
		else
			return true;
	}
	
	public boolean isTable() {
		System.out.println("isTable sur " + name);
	
		CustomType	c = getCustomType();
		if (c != null && c.isSequence())
			return true;
		else {
			if (parent != null)
				if (parent.isParentTable())
					return true;
		}
		return false;
	}
	
	private boolean isParentTable() {
		CustomType	c = getCustomType();
		if (c != null && c.isSequence())
			return true;
		else
			return false;	
	}
	
	public MibNode getParent() {
		return parent;
	}
	
	public MibNode[] getChilds() {
		MibNode[]	ret = new MibNode[childs.size()];
		for (int i=0; i<childs.size(); i++) {
			ret[i] = childs.get(i);
		}
		return ret;
	}
	
	public int deep() {
		if (parent == null)
			return 1;
		else
			return 1 + parent.deep();
	}
	
	public String toString() {
		if (parent == null)
			return name;
		else
			return (parent.toString() + "." + name);
	}

	public String toIntString() {
		if (parent == null)
			return "." + index;
		else
			return (parent.toIntString() + "." + index);
	}
	
	public void addChild (int i, MibNode m) {
		childs.add(m);
	}
	
	public void display (int depth) {
		for (int i=0; i<depth; i++) {
			System.out.print("   ");
		}
		System.out.println(name);
		
		Iterator<MibNode> ite = childs.iterator();
		while (ite.hasNext())
			ite.next().display(depth+1);
	}
	
	public MibNode getChild (String nm) throws MibNodeNotFoundException {
		//System.out.println("--== Recherche de " + name + " dans " + getName() + " de type " + nodeType + " ==--");

		switch (nodeType) {
		case NODE:
			Iterator<MibNode>	i = childs.iterator();
			MibNode				n;
		
			while (i.hasNext()) {
				n = i.next();
				//System.out.println("--> " + n.getIndex() + " " + n.getName());
			
				if (nm.compareTo(n.getName())==0) {
					return n;
				} else {
					String s = "" + n.getIndex();
					if (nm.compareTo(s)==0) {
						return n;
					}
				}
			}
			throw new MibNodeNotFoundException(toString());
//			break;
		case LEAF:
//			System.out.println("getChild dans " + name);
			CustomType	c = getCustomType();
			if (c != null && c.isSequence()) {
				CustomType.SubType st = c.getSubType(nm);
				return new MibTabNode(this, st.getIndex(), st.getName(), st.getType());
			} else {
				//System.out.println("Simple feuille");
				if (nm.compareTo("0")==0) {
					return new MibVirtualNode(this, 0, name+".0");
				} else {
					throw new MibNodeNotFoundException(toString());
				}
			}
		case VIRTUAL_NODE:
			throw new MibNodeNotFoundException(toString());
//			break;
		case TAB_NODE:
			int z;
			try {
				z = Integer.parseInt(nm);
				//if (z <= 0) throw new MibNodeNotFoundException(toString());
				return new MibTabNode(this, z, nm, type);
			} catch (NumberFormatException e) {
				throw new MibNodeNotFoundException(toString());
			}
			
		}
		return null;
	}
	
	public CustomType getCustomType() {
		//System.out.println("getCustomType sur " + name + " - " + type);
	
		if (type == null) return null;
	
		StringTokenizer	tok = new StringTokenizer(type);
		String			s = tok.nextToken();
		CustomType		ret = null;
		
		//System.out.println("getCustomType avec type : " + s);
		
		if (s.compareTo("INTEGER")==0) {
			ret = new CustomType("INTEGER", CustomType.INTEGER);
		} else if (s.compareTo("OBJECT")==0) {
			ret = new CustomType("OBJECT", CustomType.OBJECT);
		} else if (s.compareTo("OCTETSTRING")==0) {
			ret = new CustomType("OCTETSTRING", CustomType.OCTETSTRING);
		} else {
			//System.out.println("name : " + name);
			ret = module.getCustomType(s);
		}
		
		return ret;
	}
}
