package reseau;

import java.util.Iterator;
import java.util.Vector;
import java.io.*;

public class MibTree {
	private	MibNode	root;
	private MibNode treeParent;
	
	private Vector<MibNode>	elements;
	
	public MibTree() {
		elements = new Vector<MibNode>();
		root = null;
	}
	
	public MibTree(MibNode r) {
		elements = new Vector<MibNode>();
		root = r;
		elements.add(root);
	}
	
	public MibNode getRoot() {
		return root;
	}
	
	public MibNode findNode (String s) {
		Iterator<MibNode>	i = elements.iterator();
		MibNode				m;
		
		//System.out.println("------ Recherche du node " + s + " -------");
		
		while (i.hasNext()) {
			m = i.next();
			//System.out.println("--> " + m.getName());
			if (m.getName().compareTo(s) == 0) {
				return m;
			}
		}
		return null;
	}
	
	public boolean insertNode (String p, int i, String o, MibModule mod) {
		MibNode parent = findNode(p);
		
		if (parent == null)
			return false;
		else {
			MibNode m = new MibNode(parent, i, o, mod);
			parent.addChild(i,m);
			elements.add(m);
			return true;
		}
	}
	
	public boolean insertRootNode (String p, int i, String o, MibModule mod) {
		MibNode parent = findNode(p);
		
		if (parent == null)
			return false;
		else {
			MibNode m = new MibNode(parent, i, o, mod);
			//parent.addChild(i,m);
			elements.add(m);
			root = m;
			treeParent = parent;
			return true;
		}
	}

	public boolean insertLeaf (String p, int i, String o, MibModule mod, String access, String status, String descr, String type) {
		MibNode parent = findNode(p);
		int		a, s;
		
		if (parent == null)
			return false;
		else {
			a = Mib.ACCESS_NA;
			if (access.compareTo("read-only") == 0) {
				a = Mib.ACCESS_RO;
			} else if (access.compareTo("read-write") == 0) {
				a = Mib.ACCESS_RW;
			}
			
			s = Mib.STATUS_CURRENT;
			if (access.compareTo("deprecated") == 0) {
				s = Mib.STATUS_DEPRECATED;
			} else if (access.compareTo("obsolete") == 0) {
				s = Mib.STATUS_OBSOLETE;
			}
		
			//MibLeaf m = new MibLeaf(parent, i, o, a, s, descr, type);
			MibNode m = new MibNode(parent, i, o, mod, a, s, descr, type);
			//System.out.println("Creation de la feuille " + o + " de parent " + parent.getName());
			parent.addChild(i,m);
			elements.add(m);
			return true;
		}
	}
	
	protected void addNode(MibNode m) {
		elements.add(m);
	}
	
	public void addToParent() {
		treeParent.addChild(root.getIndex(), root);
	}
	
	public void displayTree() {
		Iterator<MibNode>	i = elements.iterator();
		MibNode				m;
		
		System.out.println("------ Affichage d'arbre -------");
		
		while (i.hasNext()) {
			m = i.next();
			System.out.println("--> " + m.getName());
		}	
	}
	
	public void display() {
		root.display(0);
	}
}
